#!/bin/bash

XSD="2.xsd"

function die {
  echo "ERROR: $@"
  exit 1
}

function check_file {
  [[ -f "$1" ]] || die "FILE '$1' not found"
}

function strip {
  s=$(cat)
  s=${s## }
  echo ${s%% }
}

check_file "$XSD"

xmllint --version 2> /dev/null\
  || die "xmllint not found. Please install it: sudo apt-get install xmllint"
xsltproc --version > /dev/null\
  || die "xsltproc not found. Please install it: sudo apt-get install xsltproc"

xmllint --noout "$XSD" || die "'$XSD' is not a valid XML file."
echo "Test 0 PASSED: '$XSD' is a valid XML file."

XML=2.1.xml
check_file $XML
xmllint --noout --schema "$XSD" "$XML"\
  || die "'$XML' is not valid with schema '$XSD'"
echo "Test 1 PASSED: '$XML' is valid for schema '$XSD'"

XML=2.2.xml
check_file $XML
xmllint --noout --schema "$XSD" "$XML"\
  || die "'$XML' is not valid with schema '$XSD'"
echo "Test 2 PASSED: '$XML' is valid for schema '$XSD'"

INVALID="2.invalid0*.xml"
NUM=$(ls $INVALID | wc -l | strip)
(( NUM >= 2 )) || die "Missing test files? Found only $NUM files matching $INVALID"
for i in $INVALID; do
  xmllint --noout "$i" 2> /dev/null || die "Bug in the auto-test file '$i'"
  xmllint --noout --schema "$XSD" "$i" 2> /dev/null \
    && die "ERROR: your schema validates test file '$i', but it shouldn't!"
done
echo "Test 3 PASSED: Your schema '$XSD' seems strict."

INVALID="2.invalid1*.xml"
NUM=$(ls $INVALID | wc -l | strip)
(( NUM >= 2 )) || die "Missing test files? Found only $NUM files matching $INVALID"
for i in $INVALID; do
  xmllint --noout "$i" 2> /dev/null || die "Bug in the auto-test file '$i'"
  xmllint --noout --schema "$XSD" "$i" 2> /dev/null \
    && die "ERROR: your schema validates test file '$i', but it shouldn't!"
done
echo "Test 4 PASSED: Your schema '$XSD' seems as strict as it can be!"

echo "Congratulations! Your schema passed all tests."
